﻿/**
* Dice by Edgars Simsons. March 13, 2009
* Copyright (c) 2009 Edgars Simsons
*/

package com.edzis{
	import caurina.transitions.Tweener;
	import net.badimon.five3D.display.Graphics3D;
	import net.badimon.five3D.display.Scene3D;
	import net.badimon.five3D.display.Shape3D;
	import net.badimon.five3D.display.Sprite3D;
	import net.badimon.five3D.geom.Matrix3D;
	import net.badimon.five3D.geom.Point3D;
	import flash.display.BlendMode;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.geom.Point;
	import flash.filters.DropShadowFilter;
	import flash.filters.BevelFilter;
	
	/**
	 * Dice ©2009 Edgars Simsons, edzis.wordpress.com
	 * Licensed under the MIT license. See the source file header for more information.
	 * <hr>
	 * 3D dice with rounded edges that has a dynamic rotation animation to the next score.
	 * The following libraries are used:
	 * - FiVe3D v.2.1.1 @link http://five3d.mathieu-badimon.com/
	 * - Tweener @link http://code.google.com/p/tweener/
	 * Note that FiVe3D requires @see fl.motion.Color to be included in the classpath. You migh want to include C:\Program Files\Adobe\Adobe Flash CS3\en\Configuration\ActionScript 3.0\Classes
	 * If you use or extend it, please, show it to me.
	 *
	 * @author Edgars Simsons, http://edzis.wordpress.com, simsons.edgars{in}gmail.com
	 * @version 1.0
	 */
	public class Terning extends Sprite {
		/**
		 * Width, height and depth of the dice
		 */
		private const SIZE					:int = 70;
		
		/**
		 * The dice can be wiewed from a particular angle
		 */
		private const DEFAULT_ROTATION_X	:int = 30;
		private const DEFAULT_ROTATION_Y	:int = 30;
		
		
		/**
		 * The dice can be wiewed from a particular camera position relative to the dice center
		 */
		private const CAMERA_X			:int = 0;
		private const CAMERA_Y			:int = -100;
		private const CAMERA_Z			:int = 600;
		
		/**
		 * Manipulates the ambient light direction vector. Used to have a slight shadow on the side of the dice when viewing iot from the side
		 */
		private const LIGHT_OFFSET_X	:int = 80;
		
		public static const SCORE			:String = "dice_score";
		
		
		private const MIN_ROTATION_COUNT	:int = 2;
		private const MAX_ROTATION_COUNT	:int = 1;
		private const BOX_WIDTH				:int = 10; //160;
		private const BOX_HEIGHT					:int = 10; //160;		
		
		private const CIRCLE_RADIUS	:int = 8;
		
		private var _score	:int;
		
		private var scene:Scene3D;
		private var dice:Sprite3D;
		private var face:Shape3D;
		private var endRotations	:Array = [];

		// Lagt på farger Ola Lie 23.06.09
		private var colorFace:int = 0xEEEEFF;
		private var colorRounded:int = 0xCCCCFF;
		private var colorDots:int = 0x000044;
		
		public function Terning():void {
			build();
		}
		
		/**
		 * Makes the dice roll to a new score. Mouse position adds slight changes to the rotation directions and ammount.
		 * @param	score Gives opportunity to force a particular score. If not set in the valid range of [1;6] it is generated on random. @default -1
		 */
		public function kast(score:int = -1):void {
			_score = (score >= 1 && score <= 6)? score : Math.ceil(Math.random() * 6);
			var mousePosX:Number = (mouseX < -BOX_WIDTH/2)? -BOX_WIDTH/2 : (mouseX > BOX_WIDTH/2)? BOX_WIDTH/2 : mouseX;
			var mousePosY:Number = (mouseY < -BOX_HEIGHT/2)? -BOX_HEIGHT/2 : (mouseY > BOX_HEIGHT/2)? BOX_HEIGHT/2 : mouseY;
			var rotCountX:int = (mousePosY - BOX_HEIGHT / 2) / BOX_HEIGHT * 2 * MAX_ROTATION_COUNT;
			var rotCountY:int = (mousePosX - BOX_WIDTH / 2) / BOX_WIDTH * 2 * MAX_ROTATION_COUNT;
			var rotCountZ:int = Math.round(-MIN_ROTATION_COUNT/2 + Math.random() * MIN_ROTATION_COUNT);
			rotCountX = (rotCountX > 0)? rotCountX + MIN_ROTATION_COUNT: rotCountX - MIN_ROTATION_COUNT;
			rotCountY = (rotCountY > 0)? rotCountY + MIN_ROTATION_COUNT: rotCountY - MIN_ROTATION_COUNT;
			
			var targetRotation:Point = endRotations[_score] as Point;
			Tweener.addTween(dice, { rotationX:360 * rotCountX + targetRotation.x, rotationY:360 * rotCountY + targetRotation.y, rotationZ:360 * rotCountZ, time:2, transition:"easeOutCubic", onComplete:animationCompleteHandler } );
		}
		
		
		private function build():void{
			scene = new Scene3D();
			scene.x = CAMERA_X;
			scene.y = CAMERA_Y;
			scene.viewDistance = CAMERA_Z;
			scene.ambientLightVector = new Point3D(-CAMERA_X + LIGHT_OFFSET_X, 80-CAMERA_Y, 0);
			scene.ambientLightIntensity = 0.2;
			addChild(scene);
			
			var transformContainer:Sprite3D = new Sprite3D();
			transformContainer.rotationX = DEFAULT_ROTATION_X;
			transformContainer.rotationY = DEFAULT_ROTATION_Y;
			transformContainer.x = -CAMERA_X;
			transformContainer.y = -CAMERA_Y;
			scene.addChild(transformContainer);
			
			
			dice = new Sprite3D();
			dice.childrenSorted = true;
			dice.scaleX = dice.scaleY = dice.scaleZ = SIZE/100;
			transformContainer.addChild(dice);
			
			var j:int;
			var jj:int;
			
			//	ROUNDED CORNERS
			for (j = 0; j < 4; j++) {
				for (jj = 0; jj < 2; jj++) {
					face = new Shape3D();
					face.graphics3D.beginFill(colorRounded, 1);
					face.graphics3D.moveToSpace(45, -45, -50);
					face.graphics3D.lineToSpace(45, -48.5, -48.5);
					face.graphics3D.lineToSpace(48.5, -45, -48.5);
					face.graphics3D.lineToSpace(45, -45, -50);
					face.graphics3D.endFill();
					face.rotationY = j * 90;
					face.rotationX = (jj == 1)? 180 : 0;
					dice.addChild(face);
					
					face = new Shape3D();
					face.graphics3D.beginFill(colorRounded, 1);
					face.graphics3D.moveToSpace(45, -48.5, -48.5);
					face.graphics3D.lineToSpace(45, -50, -45);
					face.graphics3D.lineToSpace(48.5, -48.5, -45);
					face.graphics3D.lineToSpace(45, -48.5, -48.5);
					face.graphics3D.endFill();
					face.rotationY = j * 90;
					face.rotationX = (jj == 1)? 180 : 0;
					dice.addChild(face);
					
					face = new Shape3D();
					face.graphics3D.beginFill(colorRounded, 1);
					face.graphics3D.moveToSpace(48.5, -45, -48.5);
					face.graphics3D.lineToSpace(48.5, -48.5, -45);
					face.graphics3D.lineToSpace(50, -45, -45);
					face.graphics3D.lineToSpace(48.5, -45, -48.5);
					face.graphics3D.endFill();
					face.rotationY = j * 90;
					face.rotationX = (jj == 1)? 180 : 0;
					dice.addChild(face);
					
					face = new Shape3D();
					face.graphics3D.beginFill(colorRounded, 1);
					face.graphics3D.moveToSpace(45, -48.5, -48.5);
					face.graphics3D.lineToSpace(48.5, -45, -48.5);
					face.graphics3D.lineToSpace(48.5, -48.5, -45);
					face.graphics3D.lineToSpace(45, -48.5, -48.5);
					face.graphics3D.endFill();
					face.rotationY = j * 90;
					face.rotationX = (jj == 1)? 180 : 0;
					dice.addChild(face);
				}
			}
			
			//	ROUNDED EDGES
			for (j = 0; j < 4; j++) {
				for (jj = 0; jj < 3; jj++) {
				//	RECT ABOWE	
					face = new Shape3D();
					face.graphics3D.beginFill(colorRounded, 1);
					face.graphics3D.moveToSpace(45,-48.5, -48.5);
					face.graphics3D.lineToSpace(45,-50, -45);
					face.graphics3D.lineToSpace(-45,-50, -45);
					face.graphics3D.lineToSpace(-45,-48.5, -48.5);
					face.graphics3D.lineToSpace(45,-48.5, -48.5);
					face.graphics3D.endFill();
					face.rotationY = j * 90;
					face.rotationX = (jj == 1)? 180 : 0;
					face.rotationZ = (jj == 2)? 90 : 0;
					dice.addChild(face);
					
					face = new Shape3D();
					face.graphics3D.beginFill(colorRounded, 1);
					face.graphics3D.moveToSpace(-45,-45, -50);
					face.graphics3D.lineToSpace(45, -45, -50);
					face.graphics3D.lineToSpace(45,-48.5, -48.5);
					face.graphics3D.lineToSpace(-45,-48.5, -48.5);
					face.graphics3D.lineToSpace(-45,-45, -50);
					face.graphics3D.endFill();
					face.rotationY = j * 90;
					face.rotationX = (jj == 1)? 180 : 0;
					face.rotationZ = (jj == 2)? 90 : 0;
					dice.addChild(face);
				}
			}
			
			//	FACES
			for (var i:int = 0; i < 6; i++) {
				face = new Shape3D();
				face.singleSided = true;
				face.flatShaded = true;
				face.graphics3D.beginFill(colorFace);
				face.graphics3D.drawRect( -45, -45, 90, 90);
				face.graphics3D.endFill();
				dice.addChild(face);
				
				switch (i+1) {
					case 1: //	FRONT
						face.graphics3D.beginFill(colorDots);
						face.graphics3D.drawCircle(0, 0, CIRCLE_RADIUS);
						face.graphics3D.endFill();
						break;

					case 2: //	LEFT
						face.graphics3D.beginFill(colorDots);
						face.graphics3D.drawCircle(-12.5, 12.5, CIRCLE_RADIUS);
						face.graphics3D.drawCircle(12.50, -12.5, CIRCLE_RADIUS);
						face.graphics3D.endFill();
						break;
						
					case 3:	//	BOTTOM
						face.graphics3D.beginFill(colorDots);
						face.graphics3D.drawCircle(-20, 20, CIRCLE_RADIUS);
						face.graphics3D.drawCircle(0, 0, CIRCLE_RADIUS);
						face.graphics3D.drawCircle(20, -20, CIRCLE_RADIUS);
						face.graphics3D.endFill();
						break;
						
					case 4:	//	TOP
						face.graphics3D.beginFill(colorDots);
						face.graphics3D.drawCircle(-18, -18, CIRCLE_RADIUS);
						face.graphics3D.drawCircle(-18, 18, CIRCLE_RADIUS);
						face.graphics3D.drawCircle(18, -18, CIRCLE_RADIUS);
						face.graphics3D.drawCircle(18, 18, CIRCLE_RADIUS);
						face.graphics3D.endFill();
						break;
						
					case 5:	//	RIGHT
						face.graphics3D.beginFill(colorDots);
						face.graphics3D.drawCircle(-20, -20, CIRCLE_RADIUS);
						face.graphics3D.drawCircle(-20, 20, CIRCLE_RADIUS);
						face.graphics3D.drawCircle(0, 0, CIRCLE_RADIUS);
						face.graphics3D.drawCircle(20, -20, CIRCLE_RADIUS);
						face.graphics3D.drawCircle(20, 20, CIRCLE_RADIUS);
						face.graphics3D.endFill();
						break;
						
					case 6:	//	BACK
						face.graphics3D.beginFill(colorDots);
						face.graphics3D.drawCircle(-16, -30, CIRCLE_RADIUS);
						face.graphics3D.drawCircle(-16, 0, CIRCLE_RADIUS);
						face.graphics3D.drawCircle(-16, 30, CIRCLE_RADIUS);
						face.graphics3D.drawCircle(16, -30, CIRCLE_RADIUS);
						face.graphics3D.drawCircle(16, 0, CIRCLE_RADIUS);
						face.graphics3D.drawCircle(16, 30, CIRCLE_RADIUS);
						face.graphics3D.endFill();
						break;
				}
				
				switch (i+1) {
					case 1: //	FRONT
						face.z = -50;
						endRotations[i + 1] = new Point(-90,0);//new Point(0, 0);
						break;
					case 2: //	LEFT
						face.rotationY = -90;
						face.x = 50;
						endRotations[i + 1] = new Point(90,-90);//new Point(0, 90);
						break;
					case 3:	//	BOTTOM
						face.rotationX = 90;
						face.y = 50;
						endRotations[i + 1] = new Point(180, 0); // new Point(-90, 90);
						break;
					case 4:	//	TOP
						face.rotationX = -90;
						face.y = -50;
						endRotations[i + 1] = new Point(0, -90);//new Point(90, 0);
						break;
					case 5:	//	RIGHT
						face.rotationY = 90;
						face.x = -50;
						endRotations[i + 1] =   new Point(270,-90);//new Point(0, -90);
						break;
					case 6:	//	BACK
						face.rotationY = -180;
						face.z = 50;
						endRotations[i + 1] = new Point(90, 0);//new Point(0, 180);
						break;
				}
				
				dice.addChild(face);
			}
		}
		
		private function animationCompleteHandler():void{
			dispatchEvent(new Event(SCORE));
		}
		
		/**
		 * The last score to which the dice had to rotate. Available just after the @see roll() command
		 */
		public function get score():int { return _score; }
		
		/**
		 *  funksjon skyggOgSkrakant
		 */
		public static function skyggeOgSkrakant():Array {
			// Skygge
			var skygge:DropShadowFilter = new DropShadowFilter();
			skygge.color = 0x000000;
			skygge.blurX = 3;
			skygge.blurY = 3;
			skygge.angle = 45;
			skygge.alpha = 1;
			skygge.distance = 3;
			
			// 3D
			var skrakant:BevelFilter = new BevelFilter();
			skrakant.angle = 45;
			skrakant.distance = 3;
			skrakant.blurX=5;
			skrakant.blurY=5;
			
			//var filtersArray:Array = new Array(skygge, skrakant);
		  return new Array(skygge, skrakant);

		} // slutt function skyggeOgSkrakant
		
	}
}